<?php

namespace Drupal\dynamic_library_loader\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for the Dynamic Library Loader module.
 */
class DynamicLibraryLoaderForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['dynamic_library_loader.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dynamic_library_loader_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Initialize rows if not already set.
    if ($form_state->get('library_mappings') === NULL) {
      $config = $this->config('dynamic_library_loader.settings');
      $form_state->set('library_mappings', $config->get('library_mappings') ?? []);
    }

    $library_mappings = $form_state->get('library_mappings');

    // Add examples at the top.
    $form['examples'] = [
      '#type' => 'markup',
      '#markup' => $this->t('<p>Examples of valid contexts:</p>
        <ul>
          <li><strong>Content Types:</strong> <em>article</em></li>
          <li><strong>Taxonomy Terms:</strong> <em>tags</em></li>
          <li><strong>Paragraphs:</strong> <em>image_gallery</em></li>
          <li><strong>Views:</strong> <em>my_view:block_1</em></li>
          <li><strong>Block Types:</strong> <em>custom_block</em></li>
          <li><strong>Node ID:</strong> <em>123</em></li>
        </ul>'),
    ];

    // Add the container for the table.
    $form['library_mappings_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'dynamic-library-mappings-wrapper'],
    ];

    // Table header.
    $form['library_mappings_wrapper']['library_mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Context Type'),
        $this->t('Context (Machine Name or Node ID)'),
        $this->t('Theme/Module'),
        $this->t('Library'),
        $this->t('Actions'),
      ],
    ];

    // Populate the table with the rows.
    foreach ($library_mappings as $delta => $entry) {
      $form['library_mappings_wrapper']['library_mappings'][$delta] = [
        'context_type' => [
          '#type' => 'select',
          '#options' => [
            'content_type' => $this->t('Content Types'),
            'taxonomy_term' => $this->t('Taxonomy Terms'),
            'paragraph' => $this->t('Paragraphs'),
            'view' => $this->t('Views'),
            'block_type' => $this->t('Block Types'),
            'node_id' => $this->t('Node ID'),
          ],
          '#default_value' => $entry['context_type'] ?? '',
        ],
        'context' => [
          '#type' => 'textfield',
          '#default_value' => $entry['context'] ?? '',
        ],
        'theme' => [
          '#type' => 'textfield',
          '#default_value' => $entry['theme'] ?? '',
        ],
        'library' => [
          '#type' => 'textfield',
          '#default_value' => $entry['library'] ?? '',
        ],
        'actions' => [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#name' => 'remove_' . $delta,
          '#submit' => [[$this, 'removeRow']],
          '#ajax' => [
            'callback' => '::ajaxRebuild',
            'wrapper' => 'dynamic-library-mappings-wrapper',
          ],
        ],
      ];
    }

    // Add the Add Row button.
    $form['library_mappings_wrapper']['add_row'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Row'),
      '#submit' => [[$this, 'addRow']],
      '#ajax' => [
        'callback' => '::ajaxRebuild',
        'wrapper' => 'dynamic-library-mappings-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Adds a new row to the library mappings.
   */
  public function addRow(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->get('library_mappings') ?? [];
    $library_mappings[] = [
      'context_type' => '',
      'context' => '',
      'theme' => '',
      'library' => '',
    ];
    $form_state->set('library_mappings', $library_mappings);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Removes a row from the library mappings.
   */
  public function removeRow(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $delta = explode('_', $trigger['#name'])[1];

    $library_mappings = $form_state->get('library_mappings') ?? [];
    unset($library_mappings[$delta]);
    $form_state->set('library_mappings', array_values($library_mappings));
    $form_state->setRebuild(TRUE);
  }

  /**
   * AJAX callback for rebuilding the form.
   */
  public function ajaxRebuild(array &$form, FormStateInterface $form_state) {
    return $form['library_mappings_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->getValue('library_mappings');
    $cleaned_mappings = [];

    // Clean up the mappings (remove empty rows, ensure proper structure).
    foreach ($library_mappings as $entry) {
      if (!empty($entry['context_type']) && !empty($entry['context']) && !empty($entry['theme']) && !empty($entry['library'])) {
        $cleaned_mappings[] = $entry;
      }
    }

    // Save the cleaned mappings to the config.
    $this->configFactory->getEditable('dynamic_library_loader.settings')
      ->set('library_mappings', $cleaned_mappings)
      ->save();

    parent::submitForm($form, $form_state);
  }
}